/* --COPYRIGHT--,BSD_EX
 * Copyright (c) 2013, Texas Instruments Incorporated
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * *  Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * *  Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * *  Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *******************************************************************************
 *
 *                       MSP432 CODE EXAMPLE DISCLAIMER
 *
 * MSP432 code examples are self-contained low-level programs that typically
 * demonstrate a single peripheral function or device feature in a highly
 * concise manner. For this the code may rely on the device's power-on default
 * register values and settings such as the clock configuration and care must
 * be taken when combining code from several examples to avoid potential side
 * effects. Also see http://www.ti.com/tool/mspdriverlib for an API functional
 * library & https://dev.ti.com/pinmux/ for a GUI approach to peripheral configuration.
 *
 * --/COPYRIGHT--*/
//******************************************************************************
//******************************************************************************
//  MSP432P401 Demo - ADC14, Sample A14, AVcc Ref, Trigger ADC from TA0.1
//
//   Description: Trigger ADC conversion of A14 200 times at 5us rate.  A trap
//                is created in the event of a system error.  This could be
//                the result in an error setting PMM core voltage or an ISR
//                taking too long and the 5us sampling rate is not satisfied.
//                For example at 12Mhz the ADC is triggered again while still
//                in the ADC ISR, this is further complicated when the timer
//                and adc isr are coincident.  Caution should be taken when
//                re-inserting send commands in timer ISR as this may
//                interfere with timing.
//
//
//                MSP432p401rpz
//             -----------------
//         /|\|              XIN|-
//          | |                 |
//          --|RST          XOUT|-
//            |                 |
//        >---|P6.1/A14     P1.0|-->
//            |       P2.4/TA0.1|-->Output to Logic Analyzer
//            |             P2.0|-->TIME_ON (Output to Logic Analyzer)
//            |             P2.1|-->TIME_OFF (Output to Logic Analyzer)
//            |             P2.2|-->ADC_SPENTO (Output to Logic Analyzer)
//
//   Built with Code Composer Studio V6.1.3
//******************************************************************************
#include "msp.h"
#include "stdint.h"

#define TA0CLK_MHZ		1

#define STEP            5*TA0CLK_MHZ
#define TIME_ON         50*TA0CLK_MHZ
#define TIME_ADC_START  0*TA0CLK_MHZ
#define TIME_OFF        250*TA0CLK_MHZ
#define TIME_ADC_END    1000*TA0CLK_MHZ
#define ADC_SPENTO      1010*TA0CLK_MHZ
#define TIME_SAVE       1200*TA0CLK_MHZ
#define TIME_RESET      10000*TA0CLK_MHZ

#define SAMPLE_SIZE		TIME_ADC_END/STEP
int microsecondi = 0;
int index_N = 0;
uint16_t values[SAMPLE_SIZE];
void IR_off()    {P4->OUT    |=  BIT2;}
void IR_on()     {P4->OUT    &= ~BIT2;}
void led_on()    {P1->OUT    |=  BIT0;}
void led_off()   {P1->OUT    &= ~BIT0;}
void TA0_on()    {TIMER_A0->CCTL[0] |=  CCIE;}
void TA0_off()   {TIMER_A0->CCTL[0] &= ~CCIE;}  //enable interrupt - does not stop timer

void error(void);

int main(void)
{
	// volatile unsigned int i;
	volatile uint16_t i;
	uint32_t currentPowerState;
    WDT_A->CTL = WDT_A_CTL_PW | WDT_A_CTL_HOLD;                 // Stop WDT

    // GPIO Setup
    P1->OUT &= ~BIT0;                           // Clear LED to start
    P1->DIR |= BIT0;                            // Set P1.0/LED to output

    P2->DIR |= BIT4;                            // Output TA0.1 on p2.4
    P2->SEL0 |= BIT4;
    P2->SEL1 &= ~BIT4;

    P2->OUT &= ~(BIT0+BIT1+BIT2);                           // Clear LED to start
    P2->DIR |= (BIT0+BIT1+BIT2);                            // Set P1.0/LED to output

//    P4->DIR |= BIT3;
//    P4->SEL0 |= BIT3;                  // MCLK
//    P4->SEL1 &= ~BIT3;

    __enable_interrupt();
    // Enable ADC interrupt in NVIC module
    P6->SEL1 |= BIT1;
    P6->SEL0 |= BIT1;    // Canale A14 (P6.1) configurato per ADC

    /*
     * Vcore and Wait state taken from example msp432p401_cs_03
     * 24Mhz Mclk and 0 wait states, per table 5.8 in datasheet
     */

    /* Step 1: Transition to VCORE Level 1: AM0_LDO --> AM1_LDO */

    /* Get current power state, if it's not AM0_LDO, error out */
    currentPowerState = PCM->CTL0 & PCM_CTL0_CPM_MASK;
    if (currentPowerState != PCM_CTL0_CPM_0)
        error();
    while ((PCM->CTL1 & PCM_CTL1_PMR_BUSY));
    PCM->CTL0 = PCM_CTL0_KEY_VAL | PCM_CTL0_AMR_1;
    while ((PCM->CTL1 & PCM_CTL1_PMR_BUSY));
    if (PCM->IFG & PCM_IFG_AM_INVALID_TR_IFG)
        error();                            // Error if transition was not successful
    if ((PCM->CTL0 & PCM_CTL0_CPM_MASK) != PCM_CTL0_CPM_1)
        error();                            // Error if device is not in AM1_LDO mode

    /* Step 2: Configure Flash wait-state to 0 for both banks 0 & 1 */
    FLCTL->BANK0_RDCTL = FLCTL->BANK0_RDCTL & (~FLCTL_BANK0_RDCTL_WAIT_MASK) | FLCTL_BANK0_RDCTL_WAIT_0;
    FLCTL->BANK1_RDCTL  = FLCTL->BANK0_RDCTL & (~FLCTL_BANK1_RDCTL_WAIT_MASK) | FLCTL_BANK1_RDCTL_WAIT_0;

    // ========== clock a 12 MHz==========
    CS->KEY = CS_KEY_VAL;                                    // Unlock CS module for register access
    CS->CTL0 = 0;
    CS->CTL0 = CS_CTL0_DCORSEL_4;        // Set DCO to 24MHz
//    CS->CTL0 = CS_CTL0_DCORSEL_3;        // Set DCO to 12MHz (nominal, center of 8-16MHz range)
    CS->CTL1 = CS_CTL1_SELA__REFOCLK | CS_CTL1_SELS__DCOCLK | CS_CTL1_SELM__DCOCLK; // Select ACLK = REFO, SMCLK = MCLK = DCO
    CS->CLKEN |= CS_CLKEN_MODOSC_EN;
    CS->KEY = 0;                         // Lock CS module from unintended accesses

    //=============ADC14============
    ADC14->CTL0 &= ~ADC14_CTL0_ENC;                           //Turn off Enable. With few exceptions, the ADC14 control bits can only be modified when ADC14ENC = 0.
    ADC14->CTL0 = ADC14_CTL0_ON | ADC14_CTL0_SHP | ADC14_CTL0_CONSEQ_2 |
    		ADC14_CTL0_SHS_1;   // Turn on ADC14, set sampling time, repeat single channel, trigger TA0.1
    // Setting the SHP bit means that the time is determined by the settings in ADC14SHT0x where the default of 0 is four clock cycles.
    ADC14->CTL1 = ADC14_CTL1_RES__14BIT;                      // Use sampling timer, 14-bit conversion results
    ADC14->MCTL[0] = ADC14_MCTLN_VRSEL_0 | ADC14_MCTLN_INCH_14;         // A14 ADC input select; Vref=AVCC=3.3V
    // with a setting of 14, the conversion takes 16 clocks, the default ADC clock is MODCLOCK, ~25Mhz
    // 20 clocks at 25Mhz -> 800ns
    ADC14->CLRIFGR0 |= ADC14_CLRIFGR0_CLRIFG0;        // clear IFG
    ADC14->IER0 |= ADC14_IER0_IE0;                    // Enable ADC conv complete interrupt
    ADC14->CTL0 |= ADC14_CTL0_ENC;

    // ==========Configurazione TimerA0, CCR0 and CCR1 ==========
    TIMER_A0->CCR[0] = STEP-1;                        // Overflow (step dell'interrupt)
    TIMER_A0->CCR[1] = STEP/2;
    TIMER_A0->CCTL[1] |= TIMER_A_CCTLN_OUTMOD_3;      // Output TA0.1 to set on CCR1(30) and reset CCR0(59),
    TIMER_A0->CTL = TIMER_A_CTL_SSEL__SMCLK | TIMER_A_CTL_ID__8;  // SMCLK, UP mode, Divisione per 8 (24 MHz / 8 = 3 MHz)
//    TIMER_A0->CTL = TIMER_A_CTL_SSEL__SMCLK | TIMER_A_CTL_ID__4;  // SMCLK, UP mode, Divisione per 8 (24 MHz / 8 = 3 MHz)
    TIMER_A0->EX0 |= TIMER_A_EX0_IDEX__3;                         // Divisione per 3 (3 MHz / 3 = 1 MHz)
    TIMER_A0->CCTL[0] &= ~TIMER_A_CCTLN_CCIFG;        // clear IFG
    TIMER_A0->CCTL[0] |= TIMER_A_CCTLN_CCIE;          // enable ta0.0 interrupt

    SCB->SCR |= SCB_SCR_SLEEPONEXIT_Msk;              // Enable sleep on exit from ISR
    __enable_interrupt();
    NVIC->ISER[0] = 1 << ((TA0_0_IRQn) & 31);
    NVIC->ISER[0] |= 1 << ((ADC14_IRQn) & 31);

    TIMER_A0->CTL |= TIMER_A_CTL_MC__UP;

    while (1)
    {
      __sleep();
      __no_operation();                     // For debugger
    }
}

void error(void)
{
    while (1);
}

// ADC14 interrupt service routine
void ADC14_IRQHandler(void)
{
	values[index_N] = ADC14->MEM[0];
	index_N++;
    if (index_N >= SAMPLE_SIZE)
    {
        TIMER_A0->CCTL[1] &= ~TIMER_A_CCTLN_OUTMOD_MASK;      // Clear output mode, stop timer trigger to ADC
        ADC14->CTL0 &= ~ADC14_CTL0_ENC;
        led_off();
        //index_N = 0;
        //microsecondi = 0;
        //TA0_off();
    }
    //P1->OUT ^= BIT0;                           // Clear LED to start
}

void TimerA0_0IsrHandler(void)
{
	TIMER_A0->CCTL[0] &= ~TIMER_A_CCTLN_CCIFG;
    /* STEP = 20 */
//    if(TIME_ADC_START < microsecondi <= TIME_ADC_END) {
//        P1->OUT |= BIT0;                           //
//        ADC14->CTL0 |= ADC14_CTL0_ENC | ADC14_CTL0_SC; // Start sampling/conversion; Enable and start conversion.
//        //send(com_LOG, "...adc...\n");
//        while(!(ADC14->IFGR0 & BIT0)); // Controlla che non abbia finito
//        values[index_N] = ADC14->MEM[0];
//        index_N++;
//        P1->OUT &= ~BIT0;                           //
//    }
    switch (microsecondi)
    {
    case TIME_ON:
        IR_on();
        P2->OUT |= BIT0;                           //
        //send(com_LOG, "LED_ON\n");
        break;
    case TIME_OFF:
        IR_off();
        P2->OUT |= BIT1;                           //
        //send(com_LOG, "LED_OFF\n");
        break;
    case ADC_SPENTO:
        P2->OUT |= BIT2;                           //
        //ADC14->CTL0 &= ~ADC14CONSEQ_0;           // this instruction has no effect, '0'
        ADC14->CTL0 &= ~ADC14_CTL0_ENC;
        break;
    case TIME_SAVE:
        //send(com_LOG, "valori_inviati\n");
    	P2->OUT &= ~(BIT0+BIT1+BIT2);
    	TIMER_A0->CTL &= ~TIMER_A_CTL_MC_MASK;  // stop timer
    	if(index_N != SAMPLE_SIZE)
    	{
    		error();
    	}
    default:
        break;
    }
    microsecondi += STEP;
    if (microsecondi >= TIME_RESET)
    {
        microsecondi = 0;
        while(1);  // trap
    }
}
